import { defineNuxtModule, addVitePlugin, addWebpackPlugin, createResolver, addComponent, addImports } from '@nuxt/kit';
import AutoImportVite from 'unplugin-auto-import/vite';
import AutoImportWp from 'unplugin-auto-import/webpack';
import ElementPlus from 'unplugin-element-plus';
import VueComponents from 'unplugin-vue-components';
import { ElementPlusResolver } from 'unplugin-vue-components/resolvers';

const name = "nuxt-element-plus-module";
const version = "0.2.0";

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "element",
    compatibility: {
      nuxt: "3"
    }
  },
  defaults: {
    autoImport: true,
    enableDarkMode: true,
    unpluginOptions: {}
  },
  async setup(options, nuxt) {
    nuxt.options.build.transpile.push("element-plus/es");
    addVitePlugin(ElementPlus.vite(options.unpluginOptions));
    addWebpackPlugin(ElementPlus.webpack(options.unpluginOptions));
    if (options.autoImport) {
      const resolver = createResolver(".");
      Object.entries(await import('element-plus')).forEach(([key, _]) => {
        if (!key.toLowerCase().startsWith("el")) {
          return;
        }
        addComponent({
          name: key,
          filePath: resolver.resolve("node_modules/element-plus/es/components"),
          export: key
        });
        addImports({
          name: key,
          from: "element-plus"
        });
      });
      const unpluginOptions = {
        resolvers: [ElementPlusResolver()],
        dts: false
      };
      addVitePlugin(VueComponents.vite(unpluginOptions));
      addVitePlugin(AutoImportVite(unpluginOptions));
      addWebpackPlugin(VueComponents.webpack(unpluginOptions));
      addWebpackPlugin(AutoImportWp(unpluginOptions));
    }
    if (options.enableDarkMode) {
      nuxt.options.css.push("element-plus/theme-chalk/src/dark/css-vars.scss");
    }
  }
});

export { module as default };
